"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOctokit = exports.testProperties = exports.getLogStdout = exports.execNotContains = exports.execContains = exports.execCalledWith = exports.spyOnSpawn = exports.spyOnExec = exports.stdoutNotContains = exports.stdoutContains = exports.stdoutCalledWith = exports.spyOnStdout = exports.testFs = exports.setChildProcessParams = exports.testChildProcess = exports.testEnv = exports.setActionEnv = void 0;
const global_1 = __importDefault(require("./global"));
const os_1 = require("os");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const github_1 = require("@actions/github");
const js_yaml_1 = require("js-yaml");
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const setActionEnv = (rootDir) => {
    const actionSetting = (0, js_yaml_1.load)(fs_1.default.readFileSync(path_1.default.resolve(rootDir, 'action.yml'), 'utf8')) || {};
    const inputs = typeof actionSetting === 'object' && typeof actionSetting['inputs'] === 'object' ? actionSetting['inputs'] : {};
    const envs = Object.keys(inputs).filter(key => 'default' in inputs[key]).map(key => ({
        key: `INPUT_${key.replace(/ /g, '_').toUpperCase()}`,
        value: `${inputs[key].default}`,
    }));
    envs.forEach(env => {
        process.env[env.key] = env.value;
    });
    return envs;
};
exports.setActionEnv = setActionEnv;
const testEnv = (rootDir) => {
    const OLD_ENV = process.env;
    beforeEach(() => {
        jest.resetModules();
        process.env = Object.assign({}, OLD_ENV);
        delete process.env.NODE_ENV;
        if (rootDir) {
            (0, exports.setActionEnv)(rootDir);
        }
    });
    afterEach(() => {
        process.env = OLD_ENV;
    });
};
exports.testEnv = testEnv;
const testChildProcess = () => {
    afterEach(() => {
        global_1.default.mockChildProcess.stdout = 'stdout';
        global_1.default.mockChildProcess.stderr = '';
        global_1.default.mockChildProcess.error = null;
        global_1.default.mockChildProcess.code = 0;
    });
};
exports.testChildProcess = testChildProcess;
const setChildProcessParams = (params) => {
    if (typeof params.stdout === 'string' || typeof params.stdout === 'function') {
        global_1.default.mockChildProcess.stdout = params.stdout;
    }
    if (typeof params.stderr === 'string' || typeof params.stderr === 'function') {
        global_1.default.mockChildProcess.stderr = params.stderr;
    }
    if (params.error instanceof Error || typeof params.error === 'function') {
        global_1.default.mockChildProcess.error = params.error;
    }
    if (typeof params.code === 'number' || typeof params.code === 'function') {
        global_1.default.mockChildProcess.code = params.code;
    }
};
exports.setChildProcessParams = setChildProcessParams;
const testFs = (defaultExists = false) => {
    let existsData = [defaultExists];
    let callback = undefined;
    let count = 0;
    let stop = false;
    const spy = [];
    const setupMock = () => {
        if (stop) {
            return;
        }
        spy.push(jest.spyOn(fs_1.default, 'writeFileSync').mockImplementation(jest.fn()));
        spy.push(jest.spyOn(fs_1.default, 'mkdirSync').mockImplementation(jest.fn()));
        spy.push(jest.spyOn(fs_1.default, 'existsSync').mockImplementation((path) => {
            if (typeof callback === 'function') {
                return callback(path);
            }
            // eslint-disable-next-line no-magic-numbers
            const result = count < existsData.length ? existsData[count] : existsData[existsData.length - 1];
            count++;
            return result;
        }));
    };
    const clearMock = () => {
        spy.forEach(spy => spy.mockRestore());
        spy.length = 0;
        callback = undefined;
        existsData = [defaultExists];
        // eslint-disable-next-line no-magic-numbers
        count = 0;
    };
    beforeEach(setupMock);
    afterEach(clearMock);
    return (flag) => {
        callback = undefined;
        stop = false;
        if (undefined === flag) {
            stop = true;
            clearMock();
        }
        else if (typeof flag === 'function') {
            callback = flag;
        }
        else if (typeof flag === 'boolean') {
            existsData = [flag];
        }
        else {
            existsData = flag;
        }
    };
};
exports.testFs = testFs;
const spyOnStdout = () => jest.spyOn(global_1.default.mockStdout, 'write');
exports.spyOnStdout = spyOnStdout;
const stdoutCalledWith = (spyOnMock, messages) => {
    expect(spyOnMock).toBeCalledTimes(messages.length);
    messages.forEach((message, index) => {
        expect(spyOnMock.mock.calls[index][0]).toBe(message + os_1.EOL);
    });
};
exports.stdoutCalledWith = stdoutCalledWith;
const stdoutContains = (spyOnMock, messages) => {
    expect(spyOnMock.mock.calls.map(value => value[0].trim())).toEqual(expect.arrayContaining(messages));
};
exports.stdoutContains = stdoutContains;
const stdoutNotContains = (spyOnMock, messages) => {
    expect(spyOnMock.mock.calls.map(value => value[0].trim())).toEqual(expect.not.arrayContaining(messages));
};
exports.stdoutNotContains = stdoutNotContains;
const spyOnExec = () => jest.spyOn(global_1.default.mockChildProcess, 'exec');
exports.spyOnExec = spyOnExec;
const spyOnSpawn = () => jest.spyOn(global_1.default.mockChildProcess, 'spawn');
exports.spyOnSpawn = spyOnSpawn;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const execCalledWith = (spyOnMock, messages) => {
    expect(spyOnMock).toBeCalledTimes(messages.length);
    messages.forEach((message, index) => {
        if (typeof message === 'string') {
            expect(spyOnMock.mock.calls[index][0]).toBe(message);
        }
        else {
            message.forEach((message, index2) => {
                if (typeof spyOnMock.mock.calls[index][index2] === 'object') {
                    expect(spyOnMock.mock.calls[index][index2]).toEqual(message);
                }
                else {
                    expect(spyOnMock.mock.calls[index][index2]).toBe(message);
                }
            });
        }
    });
};
exports.execCalledWith = execCalledWith;
const execContains = (spyOnMock, messages) => {
    expect(spyOnMock.mock.calls.map(value => value[0])).toEqual(expect.arrayContaining(messages));
};
exports.execContains = execContains;
const execNotContains = (spyOnMock, messages) => {
    expect(spyOnMock.mock.calls.map(value => value[0])).toEqual(expect.not.arrayContaining(messages));
};
exports.execNotContains = execNotContains;
// eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/explicit-module-boundary-types
const getLogStdout = (value, prefix = '') => prefix + JSON.stringify(value, null, '\t');
exports.getLogStdout = getLogStdout;
// eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/explicit-module-boundary-types
const testProperties = (object, checks) => {
    expect(typeof object).toBe('object');
    Object.keys(checks).forEach(key => {
        expect(object).toHaveProperty(key);
        if (typeof checks[key] === 'object') {
            expect(object[key]).toEqual(checks[key]);
        }
        else {
            expect(object[key]).toBe(checks[key]);
        }
    });
};
exports.testProperties = testProperties;
const getOctokit = (token) => (0, github_1.getOctokit)(token !== null && token !== void 0 ? token : 'test-token');
exports.getOctokit = getOctokit;
